@@ -20,6 +20,7 @@
 #include "autotune.h"
 #include "benchmark.h"
 #include "bitmap.h"
+#include "bridges.h"
 #include "combinator.h"
 #include "cpt.h"
 #include "debugfile.h"
@@ -131,6 +132,7 @@ static int inner2_loop (hashcat_ctx_t *hashcat_ctx)
   status_ctx->words_base = status_ctx->words_cnt / amplifier_cnt;
 
   EVENT (EVENT_CALCULATED_WORDS_BASE);
+  EVENT (EVENT_CALCULATED_WORDS_CNT);
 
   if (user_options->keyspace == true)
   {
@@ -148,11 +150,23 @@ static int inner2_loop (hashcat_ctx_t *hashcat_ctx)
     return -1;
   }
 
-  const u64 progress_restored = status_ctx->words_off * amplifier_cnt;
-
-  for (u32 i = 0; i < hashes->salts_cnt; i++)
+  if (user_options->attack_mode == ATTACK_MODE_ASSOCIATION)
   {
-    status_ctx->words_progress_restored[i] = progress_restored;
+    const u64 progress_restored = 1 * amplifier_cnt;
+
+    for (u32 i = 0; i < status_ctx->words_off; i++)
+    {
+      status_ctx->words_progress_restored[i] = progress_restored;
+    }
+  }
+  else
+  {
+    const u64 progress_restored = status_ctx->words_off * amplifier_cnt;
+
+    for (u32 i = 0; i < hashes->salts_cnt; i++)
+    {
+      status_ctx->words_progress_restored[i] = progress_restored;
+    }
   }
 
   #ifdef WITH_BRAIN
@@ -199,6 +213,66 @@ static int inner2_loop (hashcat_ctx_t *hashcat_ctx)
 
   hc_thread_wait (backend_ctx->backend_devices_cnt, c_threads);
 
+  // check for any autotune failures
+  // by default, skipping device on error
+  // using --force, accel/loops/threads min values are used instead of skipping
+
+  int at_err = 0;
+
+  for (int backend_devices_idx = 0; backend_devices_idx < backend_ctx->backend_devices_cnt; backend_devices_idx++)
+  {
+    if (backend_ctx->enabled == false) continue;
+
+    hc_device_param_t *device_param = backend_ctx->devices_param + backend_devices_idx;
+
+    if (device_param->skipped == true) continue;
+
+    if (device_param->skipped_warning == true) continue;
+
+    if (device_param->at_status == AT_STATUS_FAILED)
+    {
+      at_err++;
+
+      if (user_options->force == false)
+      {
+        event_log_warning (hashcat_ctx, "* Device #%u: skipped, due to kernel autotune failure (%d).", device_param->device_id + 1, device_param->at_rc);
+
+        device_param->skipped = true;
+
+        // update counters
+
+        if (device_param->is_hip == true)    backend_ctx->hip_devices_active--;
+        if (device_param->is_cuda == true)   backend_ctx->cuda_devices_active--;
+        if (device_param->is_opencl == true) backend_ctx->opencl_devices_active--;
+
+        backend_ctx->backend_devices_active--;
+      }
+      else
+      {
+        event_log_warning (hashcat_ctx, "* Device #%u: detected kernel autotune failure (%d), min values will be used", device_param->device_id + 1, device_param->at_rc);
+      }
+    }
+  }
+
+  if (at_err > 0)
+  {
+    event_log_warning (hashcat_ctx, NULL);
+
+    if (user_options->force == false)
+    {
+      // if all enabled devices fail, abort session
+      if (backend_ctx->backend_devices_active <= 0)
+      {
+        event_log_error (hashcat_ctx, "Aborting session due to kernel autotune failures, for all active devices.");
+
+        event_log_warning (hashcat_ctx, "You can use --force to override this, but do not report related errors.");
+        event_log_warning (hashcat_ctx, NULL);
+
+        return -10;
+      }
+    }
+  }
+
   EVENT (EVENT_AUTOTUNE_FINISHED);
 
   /**
@@ -222,6 +296,16 @@ static int inner2_loop (hashcat_ctx_t *hashcat_ctx)
     loopback_write_open (hashcat_ctx);
   }
 
+  /**
+   * Set time for --bypass-delay and start point for --bypass-threshold
+   */
+
+  if (user_options->bypass_delay_chgd == true)
+  {
+    time (&status_ctx->timer_bypass_start);
+    status_ctx->bypass_digests_done_new = hashcat_ctx->hashes->digests_done_new;
+  }
+
   /**
    * Prepare cracking stats
    */
@@ -292,12 +376,12 @@ static int inner2_loop (hashcat_ctx_t *hashcat_ctx)
   if (status_ctx->devices_status == STATUS_EXHAUSTED)
   {
     // the options speed-only and progress-only cause hashcat to abort quickly.
-    // therefore, they will end up (if no other error occured) as STATUS_EXHAUSTED.
+    // therefore, they will end up (if no other error occurred) as STATUS_EXHAUSTED.
     // however, that can create confusion in hashcats RC, because exhausted translates to RC = 1.
-    // but then having RC = 1 does not match our expection if we use for speed-only and progress-only.
+    // but then having RC = 1 does not match our exception if we use for speed-only and progress-only.
     // to get hashcat to return RC = 0 we have to set it to CRACKED or BYPASS
     // note: other options like --show, --left, --benchmark, --keyspace, --backend-info, etc.
-    // not not reach this section of the code, they've returned already with rc 0.
+    // do not reach this section of the code, they've returned already with rc 0.
 
     if ((user_options->speed_only == true) || (user_options->progress_only == true))
     {
@@ -323,6 +407,10 @@ static int inner2_loop (hashcat_ctx_t *hashcat_ctx)
 
   status_ctx->accessible = false;
 
+  // update newly cracked hashes per session
+
+  logfile_sub_uint (hashes->digests_done_new);
+
   EVENT (EVENT_CRACKER_FINISHED);
 
   // mark sub logfile
@@ -471,7 +559,7 @@ static int outer_loop (hashcat_ctx_t *hashcat_ctx)
 
   if (module_ctx->module_deprecated_notice != MODULE_DEFAULT)
   {
-    if (user_options->deprecated_check_disable == false)
+    if (user_options->deprecated_check == true)
     {
       if ((user_options->show == true) || (user_options->left == true))
       {
@@ -539,7 +627,7 @@ static int outer_loop (hashcat_ctx_t *hashcat_ctx)
    * potfile removes
    */
 
-  if (user_options->potfile_disable == false)
+  if (user_options->potfile == true)
   {
     EVENT (EVENT_POTFILE_REMOVE_PARSE_PRE);
 
@@ -651,6 +739,12 @@ static int outer_loop (hashcat_ctx_t *hashcat_ctx)
 
   if (hashes_init_selftest (hashcat_ctx) == -1) return -1;
 
+  /**
+   * load hashes, post automatisation
+   */
+
+  if (hashes_init_stage5 (hashcat_ctx) == -1) return -1;
+
   /**
    * load hashes, benchmark
    */
@@ -692,13 +786,13 @@ static int outer_loop (hashcat_ctx_t *hashcat_ctx)
   if (straight_ctx_init (hashcat_ctx) == -1) return -1;
 
   /**
-   * straight mode init
+   * combinator mode init
    */
 
   if (combinator_ctx_init (hashcat_ctx) == -1) return -1;
 
   /**
-   * charsets : keep them together for more easy maintainnce
+   * charsets : keep them together for more easy maintenance
    */
 
   if (mask_ctx_init (hashcat_ctx) == -1) return -1;
@@ -711,7 +805,7 @@ static int outer_loop (hashcat_ctx_t *hashcat_ctx)
   {
     if ((mask_ctx->masks_cnt > 1) || (straight_ctx->dicts_cnt > 1))
     {
-      event_log_error (hashcat_ctx, "Use of --skip/--limit is not supported with --increment or mask files.");
+      event_log_error (hashcat_ctx, "Use of --skip/--limit is not supported with --increment, mask files, multiple dictionaries, or --stdout.");
 
       return -1;
     }
@@ -731,6 +825,17 @@ static int outer_loop (hashcat_ctx_t *hashcat_ctx)
     }
   }
 
+  /**
+   * prevent the user from using -m/--hash-type together with --stdout
+   */
+
+  if (user_options->hash_mode_chgd == true && user_options->stdout_flag == true)
+  {
+    event_log_error (hashcat_ctx, "Use of -m/--hash-type is not supported with --stdout.");
+
+    return -1;
+  }
+
   /**
    * status progress init; needs hashes that's why we have to do it here and separate from status_ctx_init
    */
@@ -749,13 +854,61 @@ static int outer_loop (hashcat_ctx_t *hashcat_ctx)
 
   EVENT (EVENT_POTFILE_NUM_CRACKED);
 
+  /**
+   * setup salts for bridges, needs to be after bridge init, but before session start
+   */
+
+  EVENT (EVENT_BRIDGES_SALT_PRE);
+
+  if (bridges_salt_prepare (hashcat_ctx) == false)
+  {
+    event_log_error (hashcat_ctx, "Bridge salt preparation for hash-mode '%u' failed.", user_options->hash_mode);
+
+    return -1;
+  }
+
+  EVENT (EVENT_BRIDGES_SALT_POST);
+
   /**
    * inform the user
    */
 
   EVENT (EVENT_BACKEND_SESSION_PRE);
 
-  if (backend_session_begin (hashcat_ctx) == -1) return -1;
+  if (backend_session_begin (hashcat_ctx) == -1)
+  {
+    if (user_options->benchmark == true)
+    {
+      if (user_options->hash_mode_chgd == false)
+      {
+        // finalize backend session
+
+        backend_session_destroy (hashcat_ctx);
+
+        // clean up
+
+        #ifdef WITH_BRAIN
+        brain_ctx_destroy       (hashcat_ctx);
+        #endif
+
+        bridges_salt_destroy    (hashcat_ctx);
+        bridges_destroy         (hashcat_ctx);
+        bitmap_ctx_destroy      (hashcat_ctx);
+        combinator_ctx_destroy  (hashcat_ctx);
+        cpt_ctx_destroy         (hashcat_ctx);
+        hashconfig_destroy      (hashcat_ctx);
+        hashes_destroy          (hashcat_ctx);
+        mask_ctx_destroy        (hashcat_ctx);
+        status_progress_destroy (hashcat_ctx);
+        straight_ctx_destroy    (hashcat_ctx);
+        wl_data_destroy         (hashcat_ctx);
+
+        return 0;
+      }
+    }
+
+    return -1;
+  }
 
   EVENT (EVENT_BACKEND_SESSION_POST);
 
@@ -806,6 +959,8 @@ static int outer_loop (hashcat_ctx_t *hashcat_ctx)
         event_log_warning (hashcat_ctx, "You can use --self-test-disable to override, but do not report related errors.");
         event_log_warning (hashcat_ctx, NULL);
 
+        backend_ctx->self_test_warnings = true;
+
         return -1;
       }
     }
@@ -910,7 +1065,12 @@ static int outer_loop (hashcat_ctx_t *hashcat_ctx)
 
   // clean up
 
+  #ifdef WITH_BRAIN
   brain_ctx_destroy       (hashcat_ctx);
+  #endif
+
+  bridges_salt_destroy    (hashcat_ctx);
+  bridges_destroy         (hashcat_ctx);
   bitmap_ctx_destroy      (hashcat_ctx);
   combinator_ctx_destroy  (hashcat_ctx);
   cpt_ctx_destroy         (hashcat_ctx);
@@ -940,8 +1100,9 @@ int hashcat_init (hashcat_ctx_t *hashcat_ctx, void (*event) (const u32, struct h
     hashcat_ctx->event = event;
   }
 
-  hashcat_ctx->brain_ctx          = (brain_ctx_t *)           hcmalloc (sizeof (brain_ctx_t));
   hashcat_ctx->bitmap_ctx         = (bitmap_ctx_t *)          hcmalloc (sizeof (bitmap_ctx_t));
+  hashcat_ctx->brain_ctx          = (brain_ctx_t *)           hcmalloc (sizeof (brain_ctx_t));
+  hashcat_ctx->bridge_ctx         = (bridge_ctx_t *)          hcmalloc (sizeof (bridge_ctx_t));
   hashcat_ctx->combinator_ctx     = (combinator_ctx_t *)      hcmalloc (sizeof (combinator_ctx_t));
   hashcat_ctx->cpt_ctx            = (cpt_ctx_t *)             hcmalloc (sizeof (cpt_ctx_t));
   hashcat_ctx->debugfile_ctx      = (debugfile_ctx_t *)       hcmalloc (sizeof (debugfile_ctx_t));
@@ -975,8 +1136,9 @@ int hashcat_init (hashcat_ctx_t *hashcat_ctx, void (*event) (const u32, struct h
 
 void hashcat_destroy (hashcat_ctx_t *hashcat_ctx)
 {
-  hcfree (hashcat_ctx->brain_ctx);
   hcfree (hashcat_ctx->bitmap_ctx);
+  hcfree (hashcat_ctx->brain_ctx);
+  hcfree (hashcat_ctx->bridge_ctx);
   hcfree (hashcat_ctx->combinator_ctx);
   hcfree (hashcat_ctx->cpt_ctx);
   hcfree (hashcat_ctx->debugfile_ctx);
@@ -1078,13 +1240,13 @@ int hashcat_session_init (hashcat_ctx_t *hashcat_ctx, const char *install_folder
     }
   }
   #endif
-  #endif
 
   /**
    * brain
    */
 
   if (brain_ctx_init (hashcat_ctx) == -1) return -1;
+  #endif
 
   /**
    * logfile
@@ -1108,7 +1270,7 @@ int hashcat_session_init (hashcat_ctx_t *hashcat_ctx, const char *install_folder
    * To help users a bit
    */
 
-  setup_environment_variables (hashcat_ctx->folder_config);
+  setup_environment_variables (hashcat_ctx->folder_config, hashcat_ctx->user_options);
 
   setup_umask ();
 
@@ -1168,18 +1330,41 @@ int hashcat_session_init (hashcat_ctx_t *hashcat_ctx, const char *install_folder
 
   if (user_options_check_files (hashcat_ctx) == -1) return -1;
 
+  /**
+   * Load bridge a bit too early actually, but we need to know the unit count so we can automatically configure virtualization for the user
+   */
+
+  EVENT (EVENT_BRIDGES_INIT_PRE);
+
+  if (bridges_init (hashcat_ctx) == false)
+  {
+    event_log_error (hashcat_ctx, "Bridge initialization for hash-mode '%u' failed.", user_options->hash_mode);
+
+    return -1;
+  }
+
+  EVENT (EVENT_BRIDGES_INIT_POST);
+
   /**
    * Init backend library loader
    */
 
+  EVENT (EVENT_BACKEND_RUNTIMES_INIT_PRE);
+
   if (backend_ctx_init (hashcat_ctx) == -1) return -1;
 
+  EVENT (EVENT_BACKEND_RUNTIMES_INIT_POST);
+
   /**
    * Init backend devices
    */
 
+  EVENT (EVENT_BACKEND_DEVICES_INIT_PRE);
+
   if (backend_ctx_devices_init (hashcat_ctx, comptime) == -1) return -1;
 
+  EVENT (EVENT_BACKEND_DEVICES_INIT_POST);
+
   /**
    * HM devices: init
    */
@@ -1277,16 +1462,23 @@ bool autodetect_hashmode_test (hashcat_ctx_t *hashcat_ctx)
 
   hashinfo_t *hash_info = (hashinfo_t *) hcmalloc (sizeof (hashinfo_t));
 
+  hash_info->dynamicx = (dynamicx_t *) hcmalloc (sizeof (dynamicx_t));
   hash_info->user = (user_t *) hcmalloc (sizeof (user_t));
   hash_info->orighash = (char *) hcmalloc (256);
   hash_info->split = (split_t *) hcmalloc (sizeof (split_t));
 
-  hash_t *hashes_buf = (hash_t *) hcmalloc (sizeof (hash_t));
+  // this is required for multi hash iterations in binary files, for instance used in -m 14600
+  #define HASHES_IN_BINARY 10
 
-  hashes_buf->digest    = digest;
-  hashes_buf->salt      = salt;
-  hashes_buf->esalt     = esalt;
-  hashes_buf->hook_salt = hook_salt;
+  hash_t *hashes_buf = (hash_t *) hccalloc (HASHES_IN_BINARY, sizeof (hash_t));
+
+  for (int i = 0; i < HASHES_IN_BINARY; i++)
+  {
+    hashes_buf[i].digest    = digest;
+    hashes_buf[i].salt      = salt;
+    hashes_buf[i].esalt     = esalt;
+    hashes_buf[i].hook_salt = hook_salt;
+  }
 
   hashes->hashes_buf     = hashes_buf;
   hashes->digests_buf    = digest;
@@ -1300,6 +1492,8 @@ bool autodetect_hashmode_test (hashcat_ctx_t *hashcat_ctx)
   {
     char *input_buf = user_options_extra->hc_hash;
 
+    if (!input_buf) return false;
+
     size_t input_len = strlen (input_buf);
 
     char  *hash_buf = NULL;
@@ -1485,6 +1679,7 @@ int hashcat_session_execute (hashcat_ctx_t *hashcat_ctx)
   logfile_ctx_t   *logfile_ctx   = hashcat_ctx->logfile_ctx;
   status_ctx_t    *status_ctx    = hashcat_ctx->status_ctx;
   user_options_t  *user_options  = hashcat_ctx->user_options;
+  backend_ctx_t   *backend_ctx   = hashcat_ctx->backend_ctx;
 
   // start logfile entry
 
@@ -1536,15 +1731,15 @@ int hashcat_session_execute (hashcat_ctx_t *hashcat_ctx)
       {
         event_log_info (hashcat_ctx, "The following %d hash-modes match the structure of your input hash:", modes_cnt);
         event_log_info (hashcat_ctx, NULL);
-        event_log_info (hashcat_ctx, "      # | Name                                                | Category");
-        event_log_info (hashcat_ctx, "  ======+=====================================================+======================================");
+        event_log_info (hashcat_ctx, "      # | Name                                                       | Category");
+        event_log_info (hashcat_ctx, "  ======+============================================================+======================================");
       }
 
       for (int i = 0; i < modes_cnt; i++)
       {
         if (user_options->machine_readable == false)
         {
-          event_log_info (hashcat_ctx, "%7u | %-51s | %s", usage_sort_buf[i].hash_mode, usage_sort_buf[i].hash_name, strhashcategory (usage_sort_buf[i].hash_category));
+          event_log_info (hashcat_ctx, "%7u | %-58s | %s", usage_sort_buf[i].hash_mode, usage_sort_buf[i].hash_name, strhashcategory (usage_sort_buf[i].hash_category));
         }
         else
         {
@@ -1586,9 +1781,9 @@ int hashcat_session_execute (hashcat_ctx_t *hashcat_ctx)
       {
         event_log_info (hashcat_ctx, "The following hash-mode match the structure of your input hash:");
         event_log_info (hashcat_ctx, NULL);
-        event_log_info (hashcat_ctx, "      # | Name                                                | Category");
-        event_log_info (hashcat_ctx, "  ======+=====================================================+======================================");
-        event_log_info (hashcat_ctx, "%7u | %-51s | %s", usage_sort_buf[0].hash_mode, usage_sort_buf[0].hash_name, strhashcategory (usage_sort_buf[0].hash_category));
+        event_log_info (hashcat_ctx, "      # | Name                                                       | Category");
+        event_log_info (hashcat_ctx, "  ======+============================================================+======================================");
+        event_log_info (hashcat_ctx, "%7u | %-58s | %s", usage_sort_buf[0].hash_mode, usage_sort_buf[0].hash_name, strhashcategory (usage_sort_buf[0].hash_category));
         event_log_info (hashcat_ctx, NULL);
       }
     }
@@ -1640,6 +1835,9 @@ int hashcat_session_execute (hashcat_ctx_t *hashcat_ctx)
 
       while ((hash_mode = benchmark_next (hashcat_ctx)) != -1)
       {
+        if ((u32) hash_mode < user_options->benchmark_min) continue;
+        if ((u32) hash_mode > user_options->benchmark_max) continue;
+
         user_options->hash_mode = hash_mode;
 
         rc_final = outer_loop (hashcat_ctx);
@@ -1688,14 +1886,27 @@ int hashcat_session_execute (hashcat_ctx_t *hashcat_ctx)
 
   if (rc_final == 0)
   {
-    if (status_ctx->devices_status == STATUS_ABORTED_FINISH)      rc_final =  5;
-    if (status_ctx->devices_status == STATUS_ABORTED_RUNTIME)     rc_final =  4;
-    if (status_ctx->devices_status == STATUS_ABORTED_CHECKPOINT)  rc_final =  3;
-    if (status_ctx->devices_status == STATUS_ABORTED)             rc_final =  2;
-    if (status_ctx->devices_status == STATUS_QUIT)                rc_final =  2;
-    if (status_ctx->devices_status == STATUS_EXHAUSTED)           rc_final =  1;
-    if (status_ctx->devices_status == STATUS_CRACKED)             rc_final =  0;
-    if (status_ctx->devices_status == STATUS_ERROR)               rc_final = -1;
+    if (status_ctx->devices_status == STATUS_ABORTED_FINISH)      rc_final = RC_FINAL_ABORT_FINISH;
+    if (status_ctx->devices_status == STATUS_ABORTED_RUNTIME)     rc_final = RC_FINAL_ABORT_RUNTIME;
+    if (status_ctx->devices_status == STATUS_ABORTED_CHECKPOINT)  rc_final = RC_FINAL_ABORT_CHECKPOINT;
+    if (status_ctx->devices_status == STATUS_ABORTED)             rc_final = RC_FINAL_ABORT;
+    if (status_ctx->devices_status == STATUS_QUIT)                rc_final = RC_FINAL_ABORT;
+    if (status_ctx->devices_status == STATUS_EXHAUSTED)           rc_final = RC_FINAL_EXHAUSTED;
+    if (status_ctx->devices_status == STATUS_CRACKED)             rc_final = RC_FINAL_OK;
+    if (status_ctx->devices_status == STATUS_ERROR)               rc_final = RC_FINAL_ERROR;
+  }
+  else if (rc_final == -1)
+  {
+    // set up the new negative status code, useful in test.sh
+    // -2 is marked as used in status_codes.txt
+    if (backend_ctx->runtime_skip_warning  == true)               rc_final = -3;
+    if (backend_ctx->memory_hit_warning    == true)               rc_final = -4;
+    if (backend_ctx->kernel_build_warning  == true)               rc_final = -5;
+    if (backend_ctx->kernel_create_warning == true)               rc_final = -6;
+    if (backend_ctx->kernel_accel_warnings == true)               rc_final = -7;
+    if (backend_ctx->extra_size_warning    == true)               rc_final = -8;
+    if (backend_ctx->mixed_warnings        == true)               rc_final = -9;
+    if (backend_ctx->self_test_warnings    == true)               rc_final = -11;
   }
 
   // special case for --stdout
@@ -1808,7 +2019,11 @@ int hashcat_get_status (hashcat_ctx_t *hashcat_ctx, hashcat_status_t *hashcat_st
 
   hashcat_status->digests_cnt                 = status_get_digests_cnt                (hashcat_ctx);
   hashcat_status->digests_done                = status_get_digests_done               (hashcat_ctx);
+  hashcat_status->digests_done_pot            = status_get_digests_done_pot           (hashcat_ctx);
+  hashcat_status->digests_done_zero           = status_get_digests_done_zero          (hashcat_ctx);
+  hashcat_status->digests_done_new            = status_get_digests_done_new           (hashcat_ctx);
   hashcat_status->digests_percent             = status_get_digests_percent            (hashcat_ctx);
+  hashcat_status->digests_percent_new         = status_get_digests_percent_new        (hashcat_ctx);
   hashcat_status->hash_target                 = status_get_hash_target                (hashcat_ctx);
   hashcat_status->hash_name                   = status_get_hash_name                  (hashcat_ctx);
   hashcat_status->guess_base                  = status_get_guess_base                 (hashcat_ctx);
@@ -1897,6 +2112,8 @@ int hashcat_get_status (hashcat_ctx_t *hashcat_ctx, hashcat_status_t *hashcat_st
     device_info->innerloop_left_dev             = status_get_innerloop_left_dev             (hashcat_ctx, device_id);
     device_info->iteration_pos_dev              = status_get_iteration_pos_dev              (hashcat_ctx, device_id);
     device_info->iteration_left_dev             = status_get_iteration_left_dev             (hashcat_ctx, device_id);
+    device_info->device_name                    = status_get_device_name                    (hashcat_ctx, device_id);
+    device_info->device_type                    = status_get_device_type                    (hashcat_ctx, device_id);
     #ifdef WITH_BRAIN
     device_info->brain_link_client_id_dev       = status_get_brain_link_client_id_dev       (hashcat_ctx, device_id);
     device_info->brain_link_status_dev          = status_get_brain_link_status_dev          (hashcat_ctx, device_id);
