@@ -30,6 +30,8 @@ int _dowildcard = -1;
 
 static void main_log_clear_line (MAYBE_UNUSED const size_t prev_len, MAYBE_UNUSED FILE *fp)
 {
+  if (!is_stdout_terminal ()) return;
+
   #if defined (_WIN)
 
   fputc ('\r', fp);
@@ -68,8 +70,6 @@ static void main_log (hashcat_ctx_t *hashcat_ctx, FILE *fp, const int loglevel)
     event_ctx->prev_len = msg_len;
   }
 
-  // color stuff pre
-
   #if defined (_WIN)
   HANDLE hConsole = GetStdHandle (STD_OUTPUT_HANDLE);
 
@@ -78,52 +78,60 @@ static void main_log (hashcat_ctx_t *hashcat_ctx, FILE *fp, const int loglevel)
   GetConsoleScreenBufferInfo (hConsole, &con_info);
 
   const int orig = con_info.wAttributes;
+  #endif
 
-  switch (loglevel)
+  // color stuff pre
+  if (is_stdout_terminal ())
   {
-    case LOGLEVEL_INFO:
-      break;
-    case LOGLEVEL_WARNING: SetConsoleTextAttribute (hConsole, 6);
-      break;
-    case LOGLEVEL_ERROR:   SetConsoleTextAttribute (hConsole, FOREGROUND_RED | FOREGROUND_INTENSITY);
-      break;
-    case LOGLEVEL_ADVICE:  SetConsoleTextAttribute (hConsole, 6);
-      break;
-  }
+  #if defined (_WIN)
+    switch (loglevel)
+    {
+      case LOGLEVEL_INFO:
+        break;
+      case LOGLEVEL_WARNING: SetConsoleTextAttribute (hConsole, 6);
+        break;
+      case LOGLEVEL_ERROR:   SetConsoleTextAttribute (hConsole, FOREGROUND_RED | FOREGROUND_INTENSITY);
+        break;
+      case LOGLEVEL_ADVICE:  SetConsoleTextAttribute (hConsole, 6);
+        break;
+    }
 
   #else
-  switch (loglevel)
-  {
-    case LOGLEVEL_INFO:                                   break;
-    case LOGLEVEL_WARNING: fwrite ("\033[33m", 5, 1, fp); break;
-    case LOGLEVEL_ERROR:   fwrite ("\033[31m", 5, 1, fp); break;
-    case LOGLEVEL_ADVICE:  fwrite ("\033[33m", 5, 1, fp); break;
-  }
+    switch (loglevel)
+    {
+      case LOGLEVEL_INFO:                                   break;
+      case LOGLEVEL_WARNING: fwrite ("\033[33m", 5, 1, fp); break;
+      case LOGLEVEL_ERROR:   fwrite ("\033[31m", 5, 1, fp); break;
+      case LOGLEVEL_ADVICE:  fwrite ("\033[33m", 5, 1, fp); break;
+    }
   #endif
+  }
 
   // finally, print
 
   fwrite (msg_buf, msg_len, 1, fp);
 
   // color stuff post
-
+  if (is_stdout_terminal ())
+  {
   #if defined (_WIN)
-  switch (loglevel)
-  {
-    case LOGLEVEL_INFO:                                              break;
-    case LOGLEVEL_WARNING: SetConsoleTextAttribute (hConsole, orig); break;
-    case LOGLEVEL_ERROR:   SetConsoleTextAttribute (hConsole, orig); break;
-    case LOGLEVEL_ADVICE:  SetConsoleTextAttribute (hConsole, orig); break;
-  }
+    switch (loglevel)
+    {
+      case LOGLEVEL_INFO:                                              break;
+      case LOGLEVEL_WARNING: SetConsoleTextAttribute (hConsole, orig); break;
+      case LOGLEVEL_ERROR:   SetConsoleTextAttribute (hConsole, orig); break;
+      case LOGLEVEL_ADVICE:  SetConsoleTextAttribute (hConsole, orig); break;
+    }
   #else
-  switch (loglevel)
-  {
-    case LOGLEVEL_INFO:                                  break;
-    case LOGLEVEL_WARNING: fwrite ("\033[0m", 4, 1, fp); break;
-    case LOGLEVEL_ERROR:   fwrite ("\033[0m", 4, 1, fp); break;
-    case LOGLEVEL_ADVICE:  fwrite ("\033[0m", 4, 1, fp); break;
-  }
+    switch (loglevel)
+    {
+      case LOGLEVEL_INFO:                                  break;
+      case LOGLEVEL_WARNING: fwrite ("\033[0m", 4, 1, fp); break;
+      case LOGLEVEL_ERROR:   fwrite ("\033[0m", 4, 1, fp); break;
+      case LOGLEVEL_ADVICE:  fwrite ("\033[0m", 4, 1, fp); break;
+    }
   #endif
+  }
 
   // eventual newline
 
@@ -146,7 +154,7 @@ static void main_log_advice (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUS
 {
   const user_options_t *user_options = hashcat_ctx->user_options;
 
-  if (user_options->advice_disable == true) return;
+  if (user_options->advice == false) return;
 
   main_log (hashcat_ctx, stdout, LOGLEVEL_ADVICE);
 }
@@ -184,12 +192,13 @@ static void main_outerloop_starting (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MA
 
   status_ctx->shutdown_outer = false;
 
-  if (user_options->hash_info      == true) return;
-  if (user_options->keyspace       == true) return;
-  if (user_options->stdout_flag    == true) return;
-  if (user_options->backend_info   == true) return;
-  if (user_options->speed_only     == true) return;
-  if (user_options->identify       == true) return;
+  if (user_options->backend_info  > 0)    return;
+  if (user_options->hash_info     > 0)    return;
+
+  if (user_options->keyspace     == true) return;
+  if (user_options->stdout_flag  == true) return;
+  if (user_options->speed_only   == true) return;
+  if (user_options->identify     == true) return;
 
   if ((user_options_extra->wordlist_mode == WL_MODE_FILE) || (user_options_extra->wordlist_mode == WL_MODE_MASK))
   {
@@ -261,10 +270,11 @@ static void main_cracker_finished (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYB
   const user_options_t       *user_options       = hashcat_ctx->user_options;
   const user_options_extra_t *user_options_extra = hashcat_ctx->user_options_extra;
 
-  if (user_options->hash_info       == true) return;
-  if (user_options->keyspace        == true) return;
-  if (user_options->backend_info    == true) return;
-  if (user_options->stdout_flag     == true) return;
+  if (user_options->backend_info  > 0)    return;
+  if (user_options->hash_info     > 0)    return;
+
+  if (user_options->keyspace     == true) return;
+  if (user_options->stdout_flag  == true) return;
 
   // if we had a prompt, clear it
 
@@ -313,6 +323,10 @@ static void main_cracker_finished (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYB
   {
     status_display (hashcat_ctx);
   }
+  else if (user_options->status_json == true)
+  {
+    status_display (hashcat_ctx);
+  }
   else
   {
     if (user_options->quiet == false)
@@ -340,7 +354,10 @@ static void main_cracker_hash_cracked (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx,
     if (outfile_ctx->filename == NULL) if (user_options->quiet == false) clear_prompt (hashcat_ctx);
   }
 
+  // color option for cracked hashes
+  if (user_options->color_cracked == true && is_stdout_terminal()) fputs("\033[0;36m", stdout);
   fwrite (buf, len,          1, stdout);
+  if (user_options->color_cracked == true && is_stdout_terminal()) fwrite("\033[0m", 4, 1, stdout);
   fwrite (EOL, strlen (EOL), 1, stdout);
 
   if ((user_options_extra->wordlist_mode == WL_MODE_FILE) || (user_options_extra->wordlist_mode == WL_MODE_MASK))
@@ -358,10 +375,22 @@ static void main_calculated_words_base (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx,
   const user_options_t *user_options = hashcat_ctx->user_options;
 
   if (user_options->keyspace == false) return;
+  if (user_options->total_candidates == true) return;
 
   event_log_info (hashcat_ctx, "%" PRIu64 "", status_ctx->words_base);
 }
 
+static void main_calculated_words_cnt (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
+{
+  const status_ctx_t   *status_ctx   = hashcat_ctx->status_ctx;
+  const user_options_t *user_options = hashcat_ctx->user_options;
+
+  if (user_options->keyspace == false) return;
+  if (user_options->total_candidates == false) return;
+
+  event_log_info (hashcat_ctx, "%" PRIu64 "", status_ctx->words_cnt);
+}
+
 static void main_potfile_remove_parse_pre (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
 {
   const user_options_t *user_options = hashcat_ctx->user_options;
@@ -380,6 +409,24 @@ static void main_potfile_remove_parse_post (MAYBE_UNUSED hashcat_ctx_t *hashcat_
   event_log_info_nn (hashcat_ctx, "Compared hashes with potfile entries");
 }
 
+static void main_rulesfiles_parse_pre (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
+{
+  const user_options_t *user_options = hashcat_ctx->user_options;
+
+  if (user_options->quiet == true) return;
+
+  event_log_info_nn (hashcat_ctx, "Loading rules. Please be patient...");
+}
+
+static void main_rulesfiles_parse_post (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
+{
+  const user_options_t *user_options = hashcat_ctx->user_options;
+
+  if (user_options->quiet == true) return;
+
+  event_log_info_nn (hashcat_ctx, "Loading rules finished");
+}
+
 static void main_potfile_hash_show (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
 {
   outfile_ctx_t *outfile_ctx = hashcat_ctx->outfile_ctx;
@@ -402,24 +449,30 @@ static void main_potfile_hash_left (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAY
 static void main_potfile_num_cracked (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
 {
   const user_options_t *user_options = hashcat_ctx->user_options;
-  const hashes_t       *hashes       = hashcat_ctx->hashes;
+  hashes_t             *hashes       = hashcat_ctx->hashes;
 
   if (user_options->quiet == true) return;
 
-  const int potfile_remove_cracks = hashes->digests_done;
-
-  if (potfile_remove_cracks > 0)
+  if (hashes->digests_done_zero == 1)
   {
-    if (potfile_remove_cracks == 1)
-    {
-      event_log_info (hashcat_ctx, "INFO: Removed 1 hash found as potfile entry or as empty hash.");
-      event_log_info (hashcat_ctx, NULL);
-    }
-    else
-    {
-      event_log_info (hashcat_ctx, "INFO: Removed %d hashes found as potfile entries or as empty hashes.", potfile_remove_cracks);
-      event_log_info (hashcat_ctx, NULL);
-    }
+    event_log_info (hashcat_ctx, "INFO: Removed hash found as empty hash.");
+    event_log_info (hashcat_ctx, NULL);
+  }
+  else if (hashes->digests_done_zero > 1)
+  {
+    event_log_info (hashcat_ctx, "INFO: Removed %d hashes found as empty hashes.", hashes->digests_done_zero);
+    event_log_info (hashcat_ctx, NULL);
+  }
+
+  if (hashes->digests_done_pot == 1)
+  {
+    event_log_info (hashcat_ctx, "INFO: Removed hash found as potfile entry.");
+    event_log_info (hashcat_ctx, NULL);
+  }
+  else if (hashes->digests_done_pot > 1)
+  {
+    event_log_info (hashcat_ctx, "INFO: Removed %d hashes found as potfile entries.", hashes->digests_done_pot);
+    event_log_info (hashcat_ctx, NULL);
   }
 }
 
@@ -429,7 +482,8 @@ static void main_potfile_all_cracked (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, M
 
   if (user_options->quiet == true) return;
 
-  event_log_info (hashcat_ctx, "INFO: All hashes found in potfile! Use --show to display them.");
+  event_log_info (hashcat_ctx, "INFO: All hashes found as potfile and/or empty entries! Use --show to display them.");
+  event_log_info (hashcat_ctx, "      For more information, see https://hashcat.net/faq/potfile");
   event_log_info (hashcat_ctx, NULL);
 }
 
@@ -450,15 +504,28 @@ static void main_outerloop_mainscreen (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx,
   {
     if (user_options->machine_readable == false)
     {
+      char buf[HCBUFSIZ_TINY] = { 0 };
+
+      size_t len = 0;
+
       if ((hashconfig->attack_exec == ATTACK_EXEC_OUTSIDE_KERNEL) && (hashconfig->is_salted == true))
       {
-        event_log_info (hashcat_ctx, "Hashmode: %d - %s (Iterations: %d)", hashconfig->hash_mode, hashconfig->hash_name, hashes[0].salts_buf[0].salt_iter);
+        len = snprintf (buf, sizeof (buf), "* Hash-Mode %d (%s) [Iterations: %d]", hashconfig->hash_mode, hashconfig->hash_name, hashes[0].salts_buf[0].salt_iter);
       }
       else
       {
-        event_log_info (hashcat_ctx, "Hashmode: %d - %s", hashconfig->hash_mode, hashconfig->hash_name);
+        len = snprintf (buf, sizeof (buf), "* Hash-Mode %d (%s)", hashconfig->hash_mode, hashconfig->hash_name);
       }
 
+      char line[HCBUFSIZ_TINY] = { 0 };
+
+      memset (line, '-', len);
+
+      line[len] = 0;
+
+      event_log_info (hashcat_ctx, "%s", line);
+      event_log_info (hashcat_ctx, "%s", buf);
+      event_log_info (hashcat_ctx, "%s", line);
       event_log_info (hashcat_ctx, NULL);
     }
   }
@@ -509,6 +576,32 @@ static void main_outerloop_mainscreen (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx,
     event_log_advice (hashcat_ctx, "will add ALL plains/collisions found, even duplicates, to the potfile.");
     event_log_advice (hashcat_ctx, NULL);
   }
+
+  if (hashconfig->potfile_disable == true && user_options->attack_mode != ATTACK_MODE_ASSOCIATION)
+  {
+    event_log_advice (hashcat_ctx, "ATTENTION! Potfile storage is disabled for this hash mode.");
+    event_log_advice (hashcat_ctx, "Passwords cracked during this session will NOT be stored to the potfile.");
+
+    if(user_options->outfile_chgd == false)
+    {
+      event_log_advice (hashcat_ctx, "Consider using -o to save cracked passwords.");
+    }
+
+    event_log_advice (hashcat_ctx, NULL);
+  }
+
+  if (user_options->attack_mode == ATTACK_MODE_ASSOCIATION)
+  {
+    event_log_advice (hashcat_ctx, "ATTENTION! Potfile read/write is disabled for this attack mode.");
+    event_log_advice (hashcat_ctx, "Passwords cracked during this session will NOT be stored to the potfile.");
+
+    if(user_options->outfile_chgd == false)
+    {
+      event_log_advice (hashcat_ctx, "Consider using -o to save cracked passwords.");
+    }
+
+    event_log_advice (hashcat_ctx, NULL);
+  }
   /**
    * Watchdog and Temperature balance
    */
@@ -556,7 +649,17 @@ static void main_backend_session_hostmem (MAYBE_UNUSED hashcat_ctx_t *hashcat_ct
 
   const u64 *hostmem = (const u64 *) buf;
 
-  event_log_info (hashcat_ctx, "Host memory required for this attack: %" PRIu64 " MB", *hostmem / (1024 * 1024));
+  u64 free_memory = 0;
+
+  if (get_free_memory (&free_memory) == false)
+  {
+    event_log_info (hashcat_ctx, "Host memory allocated for this attack: %" PRIu64 " MB", *hostmem / (1024 * 1024));
+  }
+  else
+  {
+    event_log_info (hashcat_ctx, "Host memory allocated for this attack: %" PRIu64 " MB (%" PRIu64 " MB free)", *hostmem / (1024 * 1024), free_memory / (1024 * 1024));
+  }
+
   event_log_info (hashcat_ctx, NULL);
 }
 
@@ -711,7 +814,7 @@ static void main_monitor_performance_hint (MAYBE_UNUSED hashcat_ctx_t *hashcat_c
   event_log_advice (hashcat_ctx, "Cracking performance lower than expected?");
   event_log_advice (hashcat_ctx, NULL);
 
-  if (user_options->optimized_kernel_enable == false)
+  if (user_options->optimized_kernel == false)
   {
     if ((hashconfig->opti_type & OPTI_TYPE_OPTIMIZED_KERNEL) == 0)
     {
@@ -733,10 +836,16 @@ static void main_monitor_performance_hint (MAYBE_UNUSED hashcat_ctx_t *hashcat_c
 
   if (user_options->slow_candidates == false)
   {
-    event_log_advice (hashcat_ctx, "* Append -S to the commandline.");
-    event_log_advice (hashcat_ctx, "  This has a drastic speed impact but can be better for specific attacks.");
-    event_log_advice (hashcat_ctx, "  Typical scenarios are a small wordlist but a large ruleset.");
-    event_log_advice (hashcat_ctx, NULL);
+    if ((user_options_extra->wordlist_mode == WL_MODE_FILE) || (user_options_extra->wordlist_mode == WL_MODE_MASK))
+    {
+      if ((user_options->attack_mode != ATTACK_MODE_HYBRID1) && (user_options->attack_mode != ATTACK_MODE_HYBRID2) && (user_options->attack_mode != ATTACK_MODE_ASSOCIATION))
+      {
+        event_log_advice (hashcat_ctx, "* Append -S to the commandline.");
+        event_log_advice (hashcat_ctx, "  This has a drastic speed impact but can be better for specific attacks.");
+        event_log_advice (hashcat_ctx, "  Typical scenarios are a small wordlist but a large ruleset.");
+        event_log_advice (hashcat_ctx, NULL);
+      }
+    }
   }
 
   event_log_advice (hashcat_ctx, "* Update your backend API runtime / driver the right way:");
@@ -776,9 +885,8 @@ static void main_monitor_temp_abort (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MA
   const user_options_t       *user_options       = hashcat_ctx->user_options;
   const user_options_extra_t *user_options_extra = hashcat_ctx->user_options_extra;
 
-  if (user_options->quiet == true) return;
 
-  if ((user_options_extra->wordlist_mode == WL_MODE_FILE) || (user_options_extra->wordlist_mode == WL_MODE_MASK))
+  if (((user_options_extra->wordlist_mode == WL_MODE_FILE) || (user_options_extra->wordlist_mode == WL_MODE_MASK)) && user_options->quiet == false)
   {
     clear_prompt (hashcat_ctx);
   }
@@ -916,9 +1024,9 @@ static void main_hashconfig_post (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE
 
   if (hashconfig->is_salted == true)
   {
-    if (hashconfig->opti_type & OPTI_TYPE_RAW_HASH)
+    if (hashconfig->opti_type & OPTI_TYPE_RAW_HASH || hashconfig->salt_type & SALT_TYPE_GENERIC)
     {
-      event_log_info (hashcat_ctx, "Minimim salt length supported by kernel: %u", hashconfig->salt_min);
+      event_log_info (hashcat_ctx, "Minimum salt length supported by kernel: %u", hashconfig->salt_min);
       event_log_info (hashcat_ctx, "Maximum salt length supported by kernel: %u", hashconfig->salt_max);
     }
   }
@@ -1059,18 +1167,119 @@ static void main_selftest_finished (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAY
   event_log_info_nn (hashcat_ctx, "Finished self-test");
 }
 
+static void main_autotune_starting (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
+{
+  const user_options_t *user_options = hashcat_ctx->user_options;
+
+  if (user_options->quiet == true) return;
+
+  event_log_info_nn (hashcat_ctx, "Starting autotune. Please be patient...");
+}
+
+static void main_autotune_finished (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
+{
+  const user_options_t *user_options = hashcat_ctx->user_options;
+
+  if (user_options->quiet == true) return;
+
+  event_log_info_nn (hashcat_ctx, "Finished autotune");
+}
+
+static void main_backend_runtimes_init_pre (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
+{
+  const user_options_t *user_options = hashcat_ctx->user_options;
+
+  if (user_options->quiet == true) return;
+
+  event_log_info_nn (hashcat_ctx, "Initializing backend runtimes. Please be patient...");
+}
+
+static void main_backend_runtimes_init_post (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
+{
+  const user_options_t *user_options = hashcat_ctx->user_options;
+
+  if (user_options->quiet == true) return;
+
+  event_log_info_nn (hashcat_ctx, "Initialized backend runtimes");
+}
+
+static void main_backend_devices_init_pre (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
+{
+  const user_options_t *user_options = hashcat_ctx->user_options;
+
+  if (user_options->quiet == true) return;
+
+  event_log_info_nn (hashcat_ctx, "Initializing backend devices. Please be patient...");
+}
+
+static void main_backend_devices_init_post (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
+{
+  const user_options_t *user_options = hashcat_ctx->user_options;
+
+  if (user_options->quiet == true) return;
+
+  event_log_info_nn (hashcat_ctx, "Initialized backend devices");
+}
+
+static void main_bridges_init_pre (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
+{
+  const user_options_t *user_options = hashcat_ctx->user_options;
+
+  if (user_options->quiet == true) return;
+
+  event_log_info_nn (hashcat_ctx, "Initializing bridges. Please be patient...");
+}
+
+static void main_bridges_init_post (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
+{
+  const user_options_t *user_options = hashcat_ctx->user_options;
+
+  if (user_options->quiet == true) return;
+
+  event_log_info_nn (hashcat_ctx, "Initialized bridges");
+}
+
+static void main_bridges_salt_pre (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
+{
+  const user_options_t *user_options = hashcat_ctx->user_options;
+
+  if (user_options->quiet == true) return;
+
+  event_log_info_nn (hashcat_ctx, "Initializing bridge salts. Please be patient...");
+}
+
+static void main_bridges_salt_post (MAYBE_UNUSED hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const void *buf, MAYBE_UNUSED const size_t len)
+{
+  const user_options_t *user_options = hashcat_ctx->user_options;
+
+  if (user_options->quiet == true) return;
+
+  event_log_info_nn (hashcat_ctx, "Initialized bridge salts");
+}
+
 static void event (const u32 id, hashcat_ctx_t *hashcat_ctx, const void *buf, const size_t len)
 {
   switch (id)
   {
+    case EVENT_AUTOTUNE_FINISHED:         main_autotune_finished         (hashcat_ctx, buf, len); break;
+    case EVENT_AUTOTUNE_STARTING:         main_autotune_starting         (hashcat_ctx, buf, len); break;
     case EVENT_SELFTEST_FINISHED:         main_selftest_finished         (hashcat_ctx, buf, len); break;
     case EVENT_SELFTEST_STARTING:         main_selftest_starting         (hashcat_ctx, buf, len); break;
     case EVENT_AUTODETECT_FINISHED:       main_autodetect_finished       (hashcat_ctx, buf, len); break;
     case EVENT_AUTODETECT_STARTING:       main_autodetect_starting       (hashcat_ctx, buf, len); break;
+    case EVENT_BACKEND_RUNTIMES_INIT_POST:main_backend_runtimes_init_post(hashcat_ctx, buf, len); break;
+    case EVENT_BACKEND_RUNTIMES_INIT_PRE: main_backend_runtimes_init_pre (hashcat_ctx, buf, len); break;
+    case EVENT_BACKEND_DEVICES_INIT_POST: main_backend_devices_init_post (hashcat_ctx, buf, len); break;
+    case EVENT_BACKEND_DEVICES_INIT_PRE:  main_backend_devices_init_pre  (hashcat_ctx, buf, len); break;
     case EVENT_BITMAP_INIT_POST:          main_bitmap_init_post          (hashcat_ctx, buf, len); break;
     case EVENT_BITMAP_INIT_PRE:           main_bitmap_init_pre           (hashcat_ctx, buf, len); break;
     case EVENT_BITMAP_FINAL_OVERFLOW:     main_bitmap_final_overflow     (hashcat_ctx, buf, len); break;
+    case EVENT_BRIDGES_INIT_POST:         main_bridges_init_post         (hashcat_ctx, buf, len); break;
+    case EVENT_BRIDGES_INIT_PRE:          main_bridges_init_pre          (hashcat_ctx, buf, len); break;
+    case EVENT_BRIDGES_SALT_POST:         main_bridges_salt_post         (hashcat_ctx, buf, len); break;
+    case EVENT_BRIDGES_SALT_PRE:          main_bridges_salt_pre          (hashcat_ctx, buf, len); break;
     case EVENT_CALCULATED_WORDS_BASE:     main_calculated_words_base     (hashcat_ctx, buf, len); break;
+    case EVENT_CALCULATED_WORDS_CNT:      main_calculated_words_cnt      (hashcat_ctx, buf, len); break;
     case EVENT_CRACKER_FINISHED:          main_cracker_finished          (hashcat_ctx, buf, len); break;
     case EVENT_CRACKER_HASH_CRACKED:      main_cracker_hash_cracked      (hashcat_ctx, buf, len); break;
     case EVENT_CRACKER_STARTING:          main_cracker_starting          (hashcat_ctx, buf, len); break;
@@ -1112,6 +1321,8 @@ static void event (const u32 id, hashcat_ctx_t *hashcat_ctx, const void *buf, co
     case EVENT_POTFILE_NUM_CRACKED:       main_potfile_num_cracked       (hashcat_ctx, buf, len); break;
     case EVENT_POTFILE_REMOVE_PARSE_POST: main_potfile_remove_parse_post (hashcat_ctx, buf, len); break;
     case EVENT_POTFILE_REMOVE_PARSE_PRE:  main_potfile_remove_parse_pre  (hashcat_ctx, buf, len); break;
+    case EVENT_RULESFILES_PARSE_POST:     main_rulesfiles_parse_post     (hashcat_ctx, buf, len); break;
+    case EVENT_RULESFILES_PARSE_PRE:      main_rulesfiles_parse_pre      (hashcat_ctx, buf, len); break;
     case EVENT_SET_KERNEL_POWER_FINAL:    main_set_kernel_power_final    (hashcat_ctx, buf, len); break;
     case EVENT_WORDLIST_CACHE_GENERATE:   main_wordlist_cache_generate   (hashcat_ctx, buf, len); break;
     case EVENT_WORDLIST_CACHE_HIT:        main_wordlist_cache_hit        (hashcat_ctx, buf, len); break;
@@ -1153,6 +1364,8 @@ int main (int argc, char **argv)
 
   if (user_options_init (hashcat_ctx) == -1)
   {
+    hashcat_destroy (hashcat_ctx);
+
     hcfree (hashcat_ctx);
 
     return -1;
@@ -1162,6 +1375,10 @@ int main (int argc, char **argv)
 
   if (user_options_getopt (hashcat_ctx, argc, argv) == -1)
   {
+    user_options_destroy (hashcat_ctx);
+
+    hashcat_destroy (hashcat_ctx);
+
     hcfree (hashcat_ctx);
 
     return -1;
@@ -1169,6 +1386,10 @@ int main (int argc, char **argv)
 
   if (user_options_sanity (hashcat_ctx) == -1)
   {
+    user_options_destroy (hashcat_ctx);
+
+    hashcat_destroy (hashcat_ctx);
+
     hcfree (hashcat_ctx);
 
     return -1;
@@ -1193,6 +1414,10 @@ int main (int argc, char **argv)
   {
     printf ("%s\n", VERSION_TAG);
 
+    user_options_destroy (hashcat_ctx);
+
+    hashcat_destroy (hashcat_ctx);
+
     hcfree (hashcat_ctx);
 
     return 0;
@@ -1206,19 +1431,19 @@ int main (int argc, char **argv)
 
   if (hashcat_session_init (hashcat_ctx, install_folder, shared_folder, argc, argv, COMPTIME) == 0)
   {
-    if (user_options->usage == true)
+    if (user_options->usage > 0)
     {
       usage_big_print (hashcat_ctx);
 
       rc_final = 0;
     }
-    else if (user_options->hash_info == true)
+    else if (user_options->hash_info > 0)
     {
       hash_info (hashcat_ctx);
 
       rc_final = 0;
     }
-    else if (user_options->backend_info == true)
+    else if (user_options->backend_info > 0)
     {
       // if this is just backend_info, no need to execute some real cracking session
 
