@@ -42,6 +42,7 @@ int get_runtime_left (const hashcat_ctx_t *hashcat_ctx)
 
 static int monitor (hashcat_ctx_t *hashcat_ctx)
 {
+  bridge_ctx_t   *bridge_ctx    = hashcat_ctx->bridge_ctx;
   hashes_t       *hashes        = hashcat_ctx->hashes;
   hwmon_ctx_t    *hwmon_ctx     = hashcat_ctx->hwmon_ctx;
   backend_ctx_t  *backend_ctx   = hashcat_ctx->backend_ctx;
@@ -55,9 +56,10 @@ static int monitor (hashcat_ctx_t *hashcat_ctx)
   bool restore_check      = false;
   bool hwmon_check        = false;
   bool performance_check  = false;
+  bool performance_warned = false;
 
   const int    sleep_time = 1;
-  const double exec_low   = 50.0;  // in ms
+  const double exec_low   = 25.0;  // in ms
   const double util_low   = 90.0;  // in percent
 
   if (user_options->runtime)
@@ -87,7 +89,10 @@ static int monitor (hashcat_ctx_t *hashcat_ctx)
 
   if (hwmon_ctx->enabled == true)
   {
-    performance_check = true; // this check simply requires hwmon to work
+    if (bridge_ctx->enabled == false)
+    {
+      performance_check = true; // this check simply requires hwmon to work
+    }
   }
 
   if ((runtime_check == false) && (remove_check == false) && (status_check == false) && (restore_check == false) && (hwmon_check == false) && (performance_check == false))
@@ -130,6 +135,20 @@ static int monitor (hashcat_ctx_t *hashcat_ctx)
 
           myabort (hashcat_ctx);
         }
+        #if defined (__APPLE__)
+        // experimental feature, check the "Sensor Graphic Hot" sensor through IOKIT/SMC to catch a GPU overtemp alarm
+        else if (temperature > (int) (user_options->hwmon_temp_abort - 10))
+        {
+          if (hm_IOKIT_SMCGetSensorGraphicHot (hashcat_ctx) == 1)
+          {
+            event_log_error (hashcat_ctx, "hm_IOKIT_SMCGetSensorGraphicHot(): Sensor Graphics HoT, GPU Overtemp");
+
+            EVENT_DATA (EVENT_MONITOR_TEMP_ABORT, &backend_devices_idx, sizeof (int));
+
+            myabort (hashcat_ctx);
+          }
+        }
+        #endif
       }
 
       for (int backend_devices_idx = 0; backend_devices_idx < backend_ctx->backend_devices_cnt; backend_devices_idx++)
@@ -137,7 +156,6 @@ static int monitor (hashcat_ctx_t *hashcat_ctx)
         hc_device_param_t *device_param = &backend_ctx->devices_param[backend_devices_idx];
 
         if (device_param->skipped == true) continue;
-
         if (device_param->skipped_warning == true) continue;
 
         const int rc_throttle = hm_get_throttle_with_devices_idx (hashcat_ctx, backend_devices_idx);
@@ -232,7 +250,7 @@ static int monitor (hashcat_ctx_t *hashcat_ctx)
       }
     }
 
-    if (performance_check == true)
+    if (performance_check == true && status_ctx->devices_status == STATUS_RUNNING && performance_warned == false)
     {
       int exec_cnt = 0;
       int util_cnt = 0;
@@ -247,7 +265,6 @@ static int monitor (hashcat_ctx_t *hashcat_ctx)
         hc_device_param_t *device_param = &backend_ctx->devices_param[backend_devices_idx];
 
         if (device_param->skipped == true) continue;
-
         if (device_param->skipped_warning == true) continue;
 
         exec_cnt++;
@@ -273,18 +290,22 @@ static int monitor (hashcat_ctx_t *hashcat_ctx)
       if (exec_cnt > 0) exec_avg = exec_total / exec_cnt;
       if (util_cnt > 0) util_avg = util_total / util_cnt;
 
-      if ((exec_avg > 0) && (exec_avg < exec_low))
+      if (((exec_avg > 0) && (exec_avg < exec_low)) || ((util_avg > 0) && (util_avg < util_low)))
       {
         performance_warnings++;
-
-        if (performance_warnings == 10) EVENT_DATA (EVENT_MONITOR_PERFORMANCE_HINT, NULL, 0);
+      }
+      else
+      {
+        if (performance_warnings > 0)
+        {
+          performance_warnings--;
+        }
       }
 
-      if ((util_avg > 0) && (util_avg < util_low))
+      if (performance_warnings == 10)
       {
-        performance_warnings++;
-
-        if (performance_warnings == 10) EVENT_DATA (EVENT_MONITOR_PERFORMANCE_HINT, NULL, 0);
+        performance_warned = true;
+        EVENT_DATA (EVENT_MONITOR_PERFORMANCE_HINT, NULL, 0);
       }
     }
 
@@ -315,6 +336,43 @@ static int monitor (hashcat_ctx_t *hashcat_ctx)
         }
       }
     }
+
+    if (user_options->bypass_delay_chgd == true)
+    {
+      time (&status_ctx->timer_bypass_cur);
+      
+      if (status_ctx->devices_status == STATUS_RUNNING)
+      {
+        // --bypass-delay check
+        if ((status_ctx->timer_bypass_cur - status_ctx->timer_bypass_start) >= user_options->bypass_delay)
+        {
+          time (&status_ctx->timer_bypass_start);
+
+          // --bypass-threshold check
+          if ((u32)(hashcat_ctx->hashes->digests_done_new - status_ctx->bypass_digests_done_new) < user_options->bypass_threshold)
+          {
+            event_log_info (hashcat_ctx, NULL);
+            event_log_info (hashcat_ctx, NULL);
+
+            bypass (hashcat_ctx);
+
+            event_log_info (hashcat_ctx, "Bypass threshold reached! Next dictionary / mask in queue selected. Bypassing current one.");
+
+            event_log_info (hashcat_ctx, NULL);
+            status_ctx->bypass_digests_done_new = 0;
+          }
+          else
+          {
+              // enough recovered to continue the session
+              status_ctx->bypass_digests_done_new = hashcat_ctx->hashes->digests_done_new;
+          }
+        }
+      }
+      else if (status_ctx->devices_status == STATUS_PAUSED)
+      {
+        status_ctx->timer_bypass_start += 1;
+      }
+    }
   }
 
   // final round of save_hash
