@@ -48,8 +48,8 @@ int straight_ctx_update_loop (hashcat_ctx_t *hashcat_ctx)
   mask_ctx_t           *mask_ctx           = hashcat_ctx->mask_ctx;
   status_ctx_t         *status_ctx         = hashcat_ctx->status_ctx;
   straight_ctx_t       *straight_ctx       = hashcat_ctx->straight_ctx;
-  user_options_extra_t *user_options_extra = hashcat_ctx->user_options_extra;
   user_options_t       *user_options       = hashcat_ctx->user_options;
+  user_options_extra_t *user_options_extra = hashcat_ctx->user_options_extra;
 
   if (user_options->attack_mode == ATTACK_MODE_STRAIGHT)
   {
@@ -91,6 +91,13 @@ int straight_ctx_update_loop (hashcat_ctx_t *hashcat_ctx)
         return -1;
       }
 
+      if (rc == -2)
+      {
+        event_log_error (hashcat_ctx, "Error reading wordlist: %s", straight_ctx->dict);
+
+        return -1;
+      }
+
       if (status_ctx->words_cnt == 0)
       {
         logfile_sub_msg ("STOP");
@@ -125,6 +132,13 @@ int straight_ctx_update_loop (hashcat_ctx_t *hashcat_ctx)
 
         return -1;
       }
+      
+      if (rc == -2)
+      {
+        event_log_error (hashcat_ctx, "Error reading wordlist: %s", combinator_ctx->dict1);
+
+        return -1;
+      }
     }
     else if (combinator_ctx->combs_mode == COMBINATOR_MODE_BASE_RIGHT)
     {
@@ -147,6 +161,13 @@ int straight_ctx_update_loop (hashcat_ctx_t *hashcat_ctx)
 
         return -1;
       }
+
+      if (rc == -2)
+      {
+        event_log_error (hashcat_ctx, "Error reading wordlist: %s", combinator_ctx->dict2);
+
+        return -1;
+      }
     }
 
     if (status_ctx->words_cnt == 0)
@@ -194,6 +215,13 @@ int straight_ctx_update_loop (hashcat_ctx_t *hashcat_ctx)
       return -1;
     }
 
+    if (rc == -2)
+    {
+      event_log_error (hashcat_ctx, "Error reading wordlist: %s", straight_ctx->dict);
+
+      return -1;
+    }
+
     if (status_ctx->words_cnt == 0)
     {
       logfile_sub_msg ("STOP");
@@ -234,9 +262,16 @@ int straight_ctx_update_loop (hashcat_ctx_t *hashcat_ctx)
         return -1;
       }
 
+      if (rc == -2)
+      {
+        event_log_error (hashcat_ctx, "Error reading wordlist: %s", straight_ctx->dict);
+
+        return -1;
+      }
+
       if ((status_ctx->words_cnt / straight_ctx->kernel_rules_cnt) != hashes->salts_cnt)
       {
-        event_log_error (hashcat_ctx, "Wordlist %s word count is not in sync with salt count", straight_ctx->dict);
+        event_log_error (hashcat_ctx, "Number of words in wordlist '%s' is not in sync with number of unique salts", straight_ctx->dict);
         event_log_error (hashcat_ctx, "Words: %" PRIu64 ", salts: %d", status_ctx->words_cnt / straight_ctx->kernel_rules_cnt, hashes->salts_cnt);
 
         return -1;
@@ -256,20 +291,21 @@ int straight_ctx_update_loop (hashcat_ctx_t *hashcat_ctx)
 
 int straight_ctx_init (hashcat_ctx_t *hashcat_ctx)
 {
-  straight_ctx_t       *straight_ctx        = hashcat_ctx->straight_ctx;
-  user_options_extra_t *user_options_extra  = hashcat_ctx->user_options_extra;
-  user_options_t       *user_options        = hashcat_ctx->user_options;
+  straight_ctx_t       *straight_ctx       = hashcat_ctx->straight_ctx;
+  user_options_t       *user_options       = hashcat_ctx->user_options;
+  user_options_extra_t *user_options_extra = hashcat_ctx->user_options_extra;
 
   straight_ctx->enabled = false;
 
-  if (user_options->hash_info      == true) return 0;
-  if (user_options->left           == true) return 0;
-  if (user_options->backend_info   == true) return 0;
-  if (user_options->show           == true) return 0;
-  if (user_options->usage          == true) return 0;
-  if (user_options->version        == true) return 0;
+  if (user_options->usage         > 0)    return 0;
+  if (user_options->backend_info  > 0)    return 0;
+  if (user_options->hash_info     > 0)    return 0;
 
-  if (user_options->attack_mode == ATTACK_MODE_BF) return 0;
+  if (user_options->left         == true) return 0;
+  if (user_options->show         == true) return 0;
+  if (user_options->version      == true) return 0;
+
+  if (user_options->attack_mode  == ATTACK_MODE_BF) return 0;
 
   straight_ctx->enabled = true;
 
@@ -289,7 +325,11 @@ int straight_ctx_init (hashcat_ctx_t *hashcat_ctx)
   {
     if (user_options->rp_files_cnt)
     {
+      EVENT (EVENT_RULESFILES_PARSE_PRE);
+
       if (kernel_rules_load (hashcat_ctx, &straight_ctx->kernel_rules_buf, &straight_ctx->kernel_rules_cnt) == -1) return -1;
+
+      EVENT (EVENT_RULESFILES_PARSE_POST);
     }
     else if (user_options->rp_gen)
     {
@@ -502,7 +542,6 @@ void straight_ctx_destroy (hashcat_ctx_t *hashcat_ctx)
   }
 
   hcfree (straight_ctx->dicts);
-
   hcfree (straight_ctx->kernel_rules_buf);
 
   memset (straight_ctx, 0, sizeof (straight_ctx_t));
